#ifndef __AXIS_H__
#define __AXIS_H__
#include "ngl/Camera.h"
#include "ngl/ShaderManager.h"
#include "ngl/TransformStack.h"
#include "ngl/Vector.h"
#include "ngl/VBOPrimitives.h"

/// @file Axis.h
/// @brief simple class to contain and draw an axis
/// @author Jonathan Macey
/// @version 1.0
/// @date 13/10/10
/// Revision History :
/// Initial Version 13/10/10
/// @class Axis
/// @brief Simple Axis drawing
class Axis
{
public:
  /// @brief ctor
  /// @param[in] _shaderName the name of the shader to invoke when drawing
  /// @parma[in] _scale uniform scale for the initial construction of the axis
  Axis(
        std::string _shaderName,
        ngl::Real _scale
      );
  /// @brief dtor
  ~Axis();

  /// @brief draw method
  /// @brief pass in the TransformStack for the shader (should have all the tx needed)
  void draw(
              ngl::Real _xRot,
              ngl::Real _yRot
           );
private :
  /// @brief the name of the shader
  std::string m_shaderName;
  /// @brief the scale of the axis
  ngl::Real m_scale;

};

#endif // AXIS_H
